/*
 * From Numerical Recipes in C (Public Domain Code)
 */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>

#define NR_END 1
#define FREE_ARG char*

/*
 * Numerical Recipes standard error handler
 */

void
nrerror(char error_text[])
{
    fprintf(stderr, "Numerical Recipes run-time error...\n");
    fprintf(stderr, "%s\n", error_text);
    fprintf(stderr, "...now exiting to system...\n");
    exit(1);
}

/*
 * Allocate an integer matrix with subscript range m[nrl..nrh][ncl..nch]
 */

int **
imatrix(long nrl, long nrh, long ncl, long nch)
{
    int **m;
    long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;

    /* allocate pointers to rows */
    m = (int **)malloc((size_t)((nrow + NR_END) * sizeof(int *)));
    if (!m)
        nrerror("allocation failure 1 in matrix()");
    m += NR_END;
    m -= nrl;

    /* allocate rows and set pointers to them */
    m[nrl] = (int *)malloc((size_t)((nrow * ncol + NR_END) * sizeof(int)));
    if (!m[nrl])
        nrerror("allocation failure 2 in matrix()");
    m[nrl] += NR_END;
    m[nrl] -= ncl;

    for (i = nrl + 1; i <= nrh; i++)
        m[i] = m[i-1] + ncol;

    /* return pointer to array of pointers to rows */
    return m;
}

/*
 * Free an int matrix allocated by imatrix()
 */
void
free_imatrix(int **m, long nrl, long nrh, long ncl, long nch)
{
    free((FREE_ARG)(m[nrl] + ncl - NR_END));
    free((FREE_ARG)(m + nrl - NR_END));
}
